"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

from django.urls import re_path

from . import views


urlpatterns = [
    re_path(r"^(?P<action>[\-_a-zA-Z0-9]{1,255})/(?P<token>[\-a-zA-Z0-9]{1,64})$",
        views.privateurl_view,
        name="privateurl"),
]



"""(C) 2013-2024 Copycat Software, LLC. All Rights Reserved."""

__version__ = "0.1.0"


import django

from django.utils.encoding import smart_str
from django.utils.translation import gettext_lazy

django.utils.encoding.smart_text = smart_str
django.utils.encoding.smart_unicode = smart_str
django.utils.translation.ugettext_lazy = gettext_lazy



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

import datetime
import random

from django.conf import settings

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse  # noqa

from django.core.validators import RegexValidator
from django.db import models, IntegrityError
from django.db import transaction
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

from ddcore.Decorators import autoconnect
from ddcore.models import BaseModel
from ddcore.Serializers import JSONEncoder

from . import settings as purl_settings


class PrivateUrlManager(models.Manager):
    """Private URL Manager."""

    def get_object_or_None(self, action, token):
        "Docstring."
        try:
            return self.select_related("user").get(
                action=action,
                token=token)
        except self.model.DoesNotExist:
            pass


@autoconnect
class PrivateUrl(BaseModel):
    """Private URL Model."""

    TOKEN_MIN_SIZE = 8
    TOKEN_MAX_SIZE = 64

    # -------------------------------------------------------------------------
    # --- Basics.
    # -------------------------------------------------------------------------
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        db_index=True,
        on_delete=models.CASCADE,
        null=True, blank=True,
        related_name="shared_private_urls",
        verbose_name=_("User"),
        help_text=_("User"))

    action = models.SlugField(
        max_length=255,
        db_index=True,
        validators=[RegexValidator(r"^[-_a-zA-Z0-9]+$")],
        verbose_name=_("Action"),
        help_text=_("Action"))
    token = models.SlugField(
        max_length=TOKEN_MAX_SIZE,
        validators=[RegexValidator(r"^[-a-zA-Z0-9]+$")],
        verbose_name=_("Token"),
        help_text=_("Token"))

    data = models.JSONField(
        null=True, blank=True,
        encoder=JSONEncoder,
        verbose_name=_("Data"),
        help_text=_("Data"))

    hits_limit = models.PositiveIntegerField(
        default=1,
        verbose_name=_("Hits Limit"),
        help_text=_("Set 0 to unlimited."))
    hit_counter = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Hits Counter"))

    # -------------------------------------------------------------------------
    # --- Significant Dates.
    # -------------------------------------------------------------------------
    first_hit = models.DateTimeField(
        null=True, blank=True,
        verbose_name=_("First Hit"),
        help_text=_("Date of the first Hit."))
    last_hit = models.DateTimeField(
        null=True, blank=True,
        verbose_name=_("Last Hit"),
        help_text=_("Date of the last Hit."))
    expire = models.DateTimeField(
        db_index=True,
        null=True, blank=True,
        verbose_name=_("Expiration Date"),
        help_text=_("Expiration Date."))

    # -------------------------------------------------------------------------
    # --- Flags.
    # -------------------------------------------------------------------------
    auto_delete = models.BooleanField(
        default=False,
        verbose_name=_("Auto-delete?"),
        help_text=_("Delete Object, if it can no longer be used?"))

    objects = PrivateUrlManager()

    class Meta:
        # db_table = "privateurl_privateurl"
        ordering = ("-created",)
        unique_together = ("action", "token")
        verbose_name = _("private url")
        verbose_name_plural = _("private urls")

    @classmethod
    def create(
            cls, action, user=None, data=None, hits_limit=1, expire=None,
            auto_delete=False, token_size=None, replace=False):
        """Create a new PrivateUrl Object.

        Parameters
        ----------
        action              :str        Action Name.
        user                :obj        User Object or `None`.
        data                :dict       Additional Data.
        hits_limit          :int        Limit the Request Hits (0 for unlimited Hits).
        expire              :datetime   Expiration Time, Date/Time, Time-delta (`None` to disable
                                        the Time Limit).
        auto_delete         :bool       Automatically remove, when URL is not available.
        token_size          :int        Length of Token (`None` for default Value from Settings).
        replace             :bool       Remove existing Object for User and Action before creating
                                        a new one.

        Returns
        -------
                            :obj        Private URL Object.

        Raises
        ------
        IntegrityError

        """
        if (
                replace and
                user):
            cls.objects.filter(action=action, user=user).delete()

        if isinstance(expire, datetime.timedelta):
            expire = timezone.now() + expire

        try:
            token = cls.generate_token(token_size=token_size)
            obj = PrivateUrl(
                user=user,
                action=action,
                token=token,
                data=data,
                hits_limit=hits_limit,
                expire=expire,
                auto_delete=auto_delete)

            with transaction.atomic():
                obj.save()

            return obj

        except IntegrityError:
            raise RuntimeError(f"Failed to create `PrivateUrl` Object ({action=}, {token_size=})")

        except Exception as exc:
            raise exc

    def is_available(self, dt=None):
        """ Return True, if the Object can be used."""
        if (
                self.expire and
                self.expire <= (dt or timezone.now())):
            return False

        if (
                self.hits_limit and
                self.hits_limit <= self.hit_counter):
            return False

        return True

    def hit_counter_inc(self):
        """Increment Hits Counter."""
        obj_is_exists = self.pk is not None
        now = timezone.now()

        self.hit_counter += 1

        if (
                self.auto_delete and
                not self.is_available(dt=now)):
            if obj_is_exists:
                self.delete()

            return

        uf = {"hit_counter", "last_hit"}

        if not self.first_hit:
            self.first_hit = now
            uf.add("first_hit")

        self.last_hit = now

        if obj_is_exists:
            self.save(update_fields=uf)

    @classmethod
    def generate_token(cls, token_size=None):
        """Generate new unique Token.

        Parameters
        ----------
        token_size          :int        Length of Token (`None` for default Value from Settings).

        Returns
        -------
        Response            :obj        Service Status.

        Raises
        ------
        AttributeError

        """
        if token_size is None:
            token_size = purl_settings.PRIVATEURL_DEFAULT_TOKEN_SIZE

        if not isinstance(token_size, int):
            raise AttributeError("Attribute `token_size` must be `int`.")

        if (
                token_size < cls.TOKEN_MIN_SIZE or
                token_size > cls.TOKEN_MAX_SIZE):
            raise AttributeError(
                f"Attribute `token_size` Value must be between {cls.TOKEN_MIN_SIZE} and "
                f"{cls.TOKEN_MAX_SIZE}.")

        return get_random_string(length=token_size)

    def get_absolute_url(self):
        """Get absolute URL."""
        return reverse(
            "{}:privateurl".format(purl_settings.PRIVATEURL_URL_NAMESPACE), kwargs={
                "action":   self.action,
                "token":    self.token,
            })



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

from importlib import import_module

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class PrivateURLConfig(AppConfig):
    """Docstring."""

    name = "privateurl"
    verbose_name = _("Django Private URL")

    def ready(self):
        """Docstring."""
        import_module("privateurl.tasks")



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

import logging

from termcolor import cprint

from django.http.response import (
    Http404,
    HttpResponseRedirect)

from ddcore.Decorators import log_default

from privateurl.models import PrivateUrl
from privateurl.signals import (
    privateurl_ok,
    privateurl_fail)


logger = logging.getLogger(__name__)


@log_default(my_logger=logger, cls_or_self=False)
def privateurl_view(request, action, token):
    """Docstring."""
    obj = PrivateUrl.objects.get_object_or_None(action, token)
    ok = False

    if (
            not obj or
            not obj.is_available()):
        results = privateurl_fail.send(
            sender=PrivateUrl,
            request=request,
            obj=obj,
            action=action)
    else:
        results = privateurl_ok.send(
            sender=PrivateUrl,
            request=request,
            obj=obj,
            action=action)
        obj.hit_counter_inc()
        ok = True

    # cprint(f"    [--- INFO ---] RESULTS  : {results}\n"
    #        f"                   OBJ      : {obj}\n"
    #        f"                   OK       : {ok}", "cyan")

    for receiver, result in results:
        if isinstance(result, dict):
            if "response" in result:
                return result["response"]

    if not ok:
        raise Http404

    return HttpResponseRedirect("/")



"""(C) 2013-2024 Copycat Software, LLC. All Rights Reserved."""

import os
import os.path

from django.utils.translation import gettext_lazy as _

from decouple import config

from . import __version__


###############################################################################
### BASIC SETTINGS                                                          ###
###############################################################################
PRODUCT_NAME = "ddaemon-django-private-url"
PRODUCT_VERSION_NUM = f"v.{__version__}"

# -----------------------------------------------------------------------------
DEBUG = bool(config("DEBUG", default=False))
DEBUG_TOOLBAR = True

# -----------------------------------------------------------------------------
# --- We have 6 Types of Environments: "local", "dev", "test", "int",
#     "staging", and "prod".
ENVIRONMENT = config("ENVIRONMENT", default="dev")
DJANGO_SETTINGS_MODULE = config("DJANGO_SETTINGS_MODULE", default="settings.dev")
PROJECT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), "..",))

# -----------------------------------------------------------------------------
ADMINS = (
    ("Artem Suvorov", "artem.suvorov@gmail.com"),
)
MANAGERS = ADMINS

# -----------------------------------------------------------------------------
DATABASES = {
    "default": {
        "ENGINE":   config("DB_ENGINE", default="django.db.backends.sqlite3", cast=str),
        "NAME":     config("DB_NAME", default="sqlite.db", cast=str),
        "USER":     config("DB_USER", default="", cast=str),
        "PASSWORD": config("DB_PASSWORD", default="", cast=str),
        "HOST":     config("DB_HOST", default="", cast=str),
        "PORT":     config("DB_PORT", default="", cast=str),
        "OPTIONS": {
            # "autocommit": True,
        }
    }
}

# -----------------------------------------------------------------------------
DOMAIN_NAME = "example.com"
ALLOWED_HOSTS = ["*"]
APPEND_SLASH = True

# -----------------------------------------------------------------------------
TIME_ZONE = "America/Los_Angeles"

# -----------------------------------------------------------------------------
LANGUAGE_CODE = "en-us"
LANGUAGES = (
    ("en",  _("English")),
    ("de",  _("Deutsch")),
    ("es",  _("Spanish")),
)

LOCALE_PATHS = (
    os.path.join(PROJECT_PATH, "locale"),
)

# -----------------------------------------------------------------------------
SITE_ID = 1

# -----------------------------------------------------------------------------
USE_I18N = True
USE_L10N = True
USE_TZ = True

# -----------------------------------------------------------------------------
ADMIN_MEDIA_PREFIX = "/static/admin/"

MEDIA_URL = "/media/"
MEDIA_ROOT = os.path.join(PROJECT_PATH, "media")

STATIC_URL = "/static/"
STATIC_ROOT = os.path.join(PROJECT_PATH, "staticserve")
STATICFILES_DIRS = (
    ("", f"{PROJECT_PATH}/static"),
)
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "django.contrib.staticfiles.finders.DefaultStorageFinder",
)

# -----------------------------------------------------------------------------
SECRET_KEY = config("SECRET_KEY", default="@zew8t_wcz!qn9=8+hheltx@&b#!x@i6ores96lhbnobr3jp*c")
SECURE_SSL_REDIRECT = bool(config("SECURE_SSL_REDIRECT", default=False))

# -----------------------------------------------------------------------------
TEMPLATES = [
    {
        "BACKEND":  "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(PROJECT_PATH, "templates/"),
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "debug":    DEBUG,
            # "loaders": [
            #     "django.template.loaders.filesystem.Loader",
            #     "django.template.loaders.app_directories.Loader",
            # ],
            "context_processors": [
                "django.template.context_processors.csrf",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'


###############################################################################
### DJANGO MIDDLEWARE CLASSES                                               ###
###############################################################################
MIDDLEWARE = (
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    # "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

ROOT_URLCONF = "urls"

WSGI_APPLICATION = "wsgi.application"

DJANGO_APPS = [
    # --- Django Apps.
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.sitemaps",
    "django.contrib.sites",
    "django.contrib.staticfiles",
]
THIRD_PARTY_APPS = [
    # --- 3rd Party Apps.
    "ddcore",
    "rangefilter",
]
PROJECT_APPS = [
    # --- Project Apps.
    "privateurl",
]
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + PROJECT_APPS

SESSION_SERIALIZER = "django.contrib.sessions.serializers.JSONSerializer"


###############################################################################
### DJANGO CACHING                                                          ###
###############################################################################
CACHES = {
    "default": {
        "BACKEND":  "django.core.cache.backends.dummy.DummyCache",
    },
    "memcached": {
        "BACKEND":  "django.core.cache.backends.memcached.PyMemcacheCache",
        # "LOCATION": "127.0.0.1:11211",
        "LOCATION": "unix:/tmp/memcached.sock",
        "OPTIONS": {
            "no_delay":         True,
            "ignore_exc":       True,
            "max_pool_size":    4,
            "use_pooling":      True,
        },
        "TIMEOUT":  60,
        "VERSION":  1,
    },
    "redis": {
        "BACKEND":  "django.core.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379",
        # "LOCATION": "redis://username:password@127.0.0.1:6379",
        "OPTIONS": {
            "db":           "10",
            "parser_class": "redis.connection.PythonParser",
            "pool_class":   "redis.BlockingConnectionPool",
        },
        "TIMEOUT":  60,
        "VERSION":  1,
    },
    "db": {
        "BACKEND":  "django.core.cache.backends.db.DatabaseCache",
        "LOCATION": "cache_table",
        "OPTIONS": {
            "MAX_ENTRIES":  1000,
        },
        "TIMEOUT":  60,
        "VERSION":  1,
    },
    "filebased": {
        "BACKEND":  "django.core.cache.backends.filebased.FileBasedCache",
        "LOCATION": "/var/tmp/django_cache",
        "OPTIONS": {
            "MAX_ENTRIES":  1000,
        },
        "TIMEOUT":  60,
        "VERSION":  1,
    },
    "locmem": {
        "BACKEND":  "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
        "OPTIONS": {
            "MAX_ENTRIES":  1000,
        },
        "TIMEOUT":  60,
        "VERSION":  1,
    },
    "dummy": {
        "BACKEND":  "django.core.cache.backends.dummy.DummyCache",
    },
}

REDIS_FILE = "/etc/uwsgi/redis_params"


###############################################################################
### DJANGO LOGGING                                                          ###
###############################################################################
LOGGING = {
    "version":                      1,
    "disable_existing_loggers":     False,
    "filters": {
        "require_debug_false": {
            "()":           "django.utils.log.RequireDebugFalse",
        },
        "require_debug_true": {
            "()":           "django.utils.log.RequireDebugTrue",
        },
    },
    "formatters": {
        "simple": {
            "format":       "[{asctime}] {levelname} {message}",
            "datefmt":      "%Y-%m-%d %H:%M:%S",
            "style":        "{",
        },
        "verbose": {
            "format":       "[{asctime}] {levelname} [{name}.{funcName}:{lineno}] {message}",
            "datefmt":      "%Y-%m-%d %H:%M:%S",
            "style":        "{",
        },
        "json": {
            "()":           "ddcore.logformat.VerboseJSONFormatter",
        },
    },
    "handlers": {
        "console": {
            "level":        "INFO",
            "filters": [
                "require_debug_true",
            ],
            "class":        "logging.StreamHandler",
            "formatter":    "simple",
        },
        "json_file": {
            "level":        "DEBUG",
            "class":        "logging.handlers.TimedRotatingFileHandler",
            "filename":     "logs/json.log",
            "when":         "midnight",
            "interval":     1,
            "backupCount":  7,
            "formatter":    "json",
        },
        "plain_file": {
            "level":        "INFO",
            "class":        "logging.handlers.TimedRotatingFileHandler",
            "filename":     "logs/plain.log",
            "when":         "midnight",
            "interval":     1,
            "backupCount":  7,
            "formatter":    "verbose",
        },
        "null": {
            "class":        "logging.NullHandler",
        },
        "mail_admins": {
            "level":        "ERROR",
            "filters": [
                "require_debug_false",
            ],
            "class":        "django.utils.log.AdminEmailHandler",
            "formatter":    "verbose",
        },
    },
    "loggers": {
        "": {
            "level":        "INFO",
            "handlers":     ["console", "json_file", "plain_file"],
            "propagate":    True,
        },
        "django": {
            "level":        "ERROR",
            "handlers":     ["console", "json_file", "plain_file"],
            "propagate":    True,
        },
        "django.request": {
            "level":        "ERROR",
            "handlers":     ["console", "json_file", "plain_file", "mail_admins"],
            "propagate":    True,
        },
        "py.warnings": {
            "handlers":     ["console", "json_file", "plain_file"],
        },
    },
}

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",
)
AUTH_USER_MODEL = "ddcore.User"


###############################################################################
### MAILING                                                                 ###
###############################################################################
EMAIL_SENDER = config("EMAIL_SENDER", default="no-reply@saneside.com")
EMAIL_SUPPORT = config("EMAIL_SUPPORT", default="support@saneside.com")


###############################################################################
### DJANGO LOGGING                                                          ###
###############################################################################
MIDDLEWARE += (
    "ddcore.middleware.DjangoRequestIDMiddleware",
    "ddcore.middleware.DjangoLoggingMiddleware",
)



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

import datetime

from django.contrib.auth import get_user_model

from django.urls import reverse, NoReverseMatch
from django.dispatch import receiver
from django.http import HttpResponse
from django.shortcuts import resolve_url
from django.test import TestCase
from django.utils.encoding import force_str

from privateurl import settings as purl_settings
from privateurl.models import PrivateUrl
from privateurl.signals import (
    privateurl_ok,
    privateurl_fail)


class TestPrivateUrl(TestCase):
    def test_manager_create(self):
        """Docstring."""
        t = PrivateUrl.create("test", expire=datetime.timedelta(days=5))
        self.assertIsInstance(t, PrivateUrl)
        self.assertIsNotNone(t.pk)
        self.assertEqual(len(t.token), purl_settings.PRIVATEURL_DEFAULT_TOKEN_SIZE)

    def test_manager_create_with_replace(self):
        """Docstring."""
        PrivateUrl.create(action="test", replace=True)
        PrivateUrl.create(action="test", replace=True)
        self.assertEqual(PrivateUrl.objects.filter(action="test").count(), 2)

        # ---------------------------------------------------------------------
        user = get_user_model().objects.create(
            username="test",
            email="test@mail.com",
            password="test")
        PrivateUrl.create(action="test", user=user, replace=True)
        PrivateUrl.create(action="test", user=user, replace=True)
        self.assertEqual(PrivateUrl.objects.filter(action="test", user=user).count(), 1)

    def test_token_size(self):
        """Docstring."""
        valid_size = int((PrivateUrl.TOKEN_MIN_SIZE + PrivateUrl.TOKEN_MAX_SIZE) / 2)
        t = PrivateUrl.create(action="test", token_size=valid_size)
        self.assertEqual(len(t.token), valid_size)

        # ---------------------------------------------------------------------
        self.assertRaises(AttributeError, PrivateUrl.create,
            action="test", token_size="test")
        self.assertRaises(AttributeError, PrivateUrl.create,
            action="test", token_size=[10, 20, 30])
        self.assertRaises(
            AttributeError, PrivateUrl.create,
            action="test", token_size=PrivateUrl.TOKEN_MIN_SIZE-1)
        self.assertRaises(
            AttributeError, PrivateUrl.create,
            action="test", token_size=PrivateUrl.TOKEN_MAX_SIZE+1)

    def test_data(self):
        """Docstring."""
        d = {
            "k":    ["v"],
        }
        t = PrivateUrl.create("test", data=d)
        self.assertIs(t.data, d)
        self.assertIs(t.data["k"], d["k"])

    def test_manager_get_object_or_None(self):
        """Docstring."""
        t = PrivateUrl.create("test")
        j = PrivateUrl.objects.get_object_or_None(t.action, t.token)
        self.assertIsInstance(j, PrivateUrl)
        self.assertEqual(t.pk, j.pk)

        n = PrivateUrl.objects.get_object_or_None("none", "none")
        self.assertIsNone(n)

    def test_get_absolute_url(self):
        """Docstring."""
        t = PrivateUrl.create("test")
        url = reverse("purl:privateurl", kwargs={
            "action":   t.action,
            "token":    t.token,
        })
        self.assertEqual(t.get_absolute_url(), url)
        self.assertEqual(resolve_url(t), url)

    def test_is_available(self):
        """Docstring."""
        t = PrivateUrl.create("test")
        self.assertTrue(t.is_available())

        t.hits_limit = 1
        t.hit_counter = 1
        self.assertFalse(t.is_available())

        t.hits_limit = 0
        self.assertTrue(t.is_available())

        t.expire = datetime.datetime(2015, 10, 10, 10, 10, 10)
        self.assertTrue(t.is_available(dt=datetime.datetime(2015, 10, 10, 10, 10, 9)))
        self.assertFalse(t.is_available(dt=datetime.datetime(2015, 10, 10, 10, 10, 11)))

    def test_hit_counter_inc(self):
        """Docstring."""
        t = PrivateUrl.create("test")
        t.hit_counter_inc()
        self.assertEqual(t.hit_counter, 1)
        self.assertIsNotNone(t.first_hit)
        self.assertIsNotNone(t.last_hit)
        self.assertEqual(t.first_hit, t.last_hit)
        self.assertIsNotNone(t.pk)

        t.hit_counter_inc()
        self.assertEqual(t.hit_counter, 2)

        j = PrivateUrl.create("test", auto_delete=True)
        j.hit_counter_inc()
        self.assertIsNone(j.pk)

    def test_long_action_name_fail(self):
        """Docstring."""
        action = "a" * 32
        a = PrivateUrl.create(action)
        b = PrivateUrl.objects.get(action=action)
        self.assertEqual(a, b)
        self.assertEqual(len(b.action), len(action))

    def test_reverse(self):
        """Docstring."""
        a = PrivateUrl.create("a" * 32)
        try:
            a.get_absolute_url()
        except NoReverseMatch as e:
            raise self.failureException("Private url reverse url error ({}).".format(e))


class TestPrivateUrlView(TestCase):
    @classmethod
    def setUpClass(cls):
        """Docstring."""
        super(TestPrivateUrlView, cls).setUpClass()

        @receiver(privateurl_ok, weak=False, dispatch_uid="ok")
        def ok(action, **kwargs):  # noqa: F841
            """Docstring."""
            if action == "test":
                return {
                    "response": HttpResponse("ok"),
                }

        @receiver(privateurl_fail, weak=False, dispatch_uid="fail")
        def fail(action, **kwargs):  # noqa: F841
            """Docstring."""
            if action == "test":
                return {
                    "response": HttpResponse("fail"),
                }

    @classmethod
    def tearDownClass(cls):
        """Docstring."""
        super(TestPrivateUrlView, cls).tearDownClass()
        privateurl_ok.disconnect(dispatch_uid="ok")
        privateurl_fail.disconnect(dispatch_uid="fail")

    def test_receivers(self):
        """Docstring."""
        t = PrivateUrl.create("test")
        response = self.client.get(t.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(force_str(response.content), "ok")

        response = self.client.get(t.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(force_str(response.content), "fail")

        t.action = "none"
        response = self.client.get(t.get_absolute_url())
        self.assertEqual(response.status_code, 404)

    def test_receivers2(self):
        """Docstring."""
        t = PrivateUrl.create("test2")
        response = self.client.get(t.get_absolute_url())
        self.assertEqual(response.status_code, 302)

        response = self.client.get(t.get_absolute_url())
        self.assertEqual(response.status_code, 404)


class TestPrivateUrlAdmin(TestCase):
    @classmethod
    def setUpClass(cls):
        """Docstring."""
        super(TestPrivateUrlAdmin, cls).setUpClass()
        PrivateUrl.create("test")
        get_user_model().objects.create_superuser("admin", "admin@site.com", "admin")

    def setUp(self):
        """Docstring."""
        self.client.login(username="admin", password="admin")

    def tearDown(self):
        """Docstring."""
        self.client.logout()

    def test_admin_list(self):
        """Docstring."""
        response = self.client.get(resolve_url("admin:privateurl_privateurl_changelist"))
        self.assertEqual(response.status_code, 200)



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

from django.contrib import admin
from django.utils.translation import gettext, ugettext_lazy as _

from .models import PrivateUrl


@admin.register(PrivateUrl)
class PrivateUrlAdmin(admin.ModelAdmin):
    list_display = ("action_with_token", "user", "created", "expire", "used", "available")
    list_filter = ("action",)
    list_select_related = ("user",)
    raw_id_fields = ("user",)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def action_with_token(self, obj):
        return "{}/{}".format(obj.action, obj.token)

    action_with_token.short_description = _("action/token")

    def used(self, obj):
        return "{} / {}".format(obj.hit_counter, obj.hits_limit or gettext("unlimited"))

    used.short_description = _("used")

    def available(self, obj):
        return obj.is_available()

    used.short_description = _("available")
    available.boolean = True



# Generated by Django 4.2.13 on 2024-12-11 20:50

import ddcore.Serializers
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PrivateUrl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('custom_data', models.JSONField(blank=True, encoder=ddcore.Serializers.JSONEncoder, null=True, verbose_name='Custom Data')),
                ('is_hidden', models.BooleanField(default=False, help_text='Is Object hidden?', verbose_name='Is hidden?')),
                ('is_private', models.BooleanField(default=False, help_text='Is Object private?', verbose_name='Is private?')),
                ('is_deleted', models.BooleanField(default=False, help_text='Is Object deleted?', verbose_name='Is deleted?')),
                ('deleted', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, null=True, verbose_name='deleted')),
                ('action', models.SlugField(help_text='Action', max_length=255, validators=[django.core.validators.RegexValidator('^[-_a-zA-Z0-9]+$')], verbose_name='Action')),
                ('token', models.SlugField(help_text='Token', max_length=64, validators=[django.core.validators.RegexValidator('^[-a-zA-Z0-9]+$')], verbose_name='Token')),
                ('data', models.JSONField(blank=True, encoder=ddcore.Serializers.JSONEncoder, help_text='Data', null=True, verbose_name='Data')),
                ('hits_limit', models.PositiveIntegerField(default=1, help_text='Set 0 to unlimited.', verbose_name='Hits Limit')),
                ('hit_counter', models.PositiveIntegerField(default=0, verbose_name='Hits Counter')),
                ('first_hit', models.DateTimeField(blank=True, help_text='Date of the first Hit.', null=True, verbose_name='First Hit')),
                ('last_hit', models.DateTimeField(blank=True, help_text='Date of the last Hit.', null=True, verbose_name='Last Hit')),
                ('expire', models.DateTimeField(blank=True, db_index=True, help_text='Expiration Date.', null=True, verbose_name='Expiration Date')),
                ('auto_delete', models.BooleanField(default=False, help_text='Delete Object, if it can no longer be used?', verbose_name='Auto-delete?')),
                ('created_by', models.ForeignKey(blank=True, help_text='User, created the Object.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user_created_%(class)s', to=settings.AUTH_USER_MODEL, verbose_name='Created by')),
                ('deleted_by', models.ForeignKey(blank=True, help_text='User, deleted the Object.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user_deleted_%(class)s', to=settings.AUTH_USER_MODEL, verbose_name='Deleted by')),
                ('modified_by', models.ForeignKey(blank=True, help_text='User, modified the Object.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user_modified_%(class)s', to=settings.AUTH_USER_MODEL, verbose_name='Modified by')),
                ('user', models.ForeignKey(blank=True, help_text='User', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shared_private_urls', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'private url',
                'verbose_name_plural': 'private urls',
                'ordering': ('-created',),
                'unique_together': {('action', 'token')},
            },
        ),
    ]



"""(C) 2013-2024 Copycat Software, LLC. All Rights Reserved."""

import os
import re

from os import path
from setuptools import (
    find_packages,
    setup)


PROJECT_PATH = path.abspath(path.dirname(__file__))
VERSION_RE = re.compile(r"""__version__ = [""]([0-9.]+((dev|rc|b)[0-9]+)?)[""]""")


with open(os.path.join(os.path.dirname(__file__), "README.md"), encoding="utf-8") as readme:
    README = readme.read()


def get_version():
    """Get Version."""
    with open(path.join(PROJECT_PATH, "privateurl", "__init__.py"), encoding="utf-8") as version:
        init = version.read()

        return VERSION_RE.search(init).group(1)


# -----------------------------------------------------------------------------
# --- Allow `setup.py` to be run from any Path.
# -----------------------------------------------------------------------------
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name="ddaemon-django-private-url",
    version=get_version(),
    packages=find_packages(),
    include_package_data=True,
    license="GPLv3 License",
    description="Django Private URL",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/asuvorov/ddaemon-django-private-url/",
    author="Artem Suvorov",
    author_email="artem.suvorov@gmail.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Plugins",
        "Framework :: Django",
        "Framework :: Django :: 4.2",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        # "License-Expression: GPL-3.0",
        # "License-File: LICENSE",
        "Natural Language :: English",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering",
        "Topic :: Security",
    ],
    install_requires=[],
    test_suite="nose.collector",
    tests_require=["nose"],
)



"""
(C) 2013-2024 Copycat Software, LLC. All Rights Reserved.
"""

from django.conf import settings


PRIVATEURL_URL_NAMESPACE = getattr(settings, "PRIVATEURL_URL_NAMESPACE", "privateurl")
PRIVATEURL_DEFAULT_TOKEN_SIZE = getattr(settings, "PRIVATEURL_DEFAULT_TOKEN_SIZE", 16)
